package com.minlinuxserver.dao.impl;



import java.util.Collections;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.minlinuxserver.util.HibernateUtil;



/**
 * @author Peter Lindblom
 * 
 */
public abstract class DaoAbstract {

	private static Logger logger = LoggerFactory.getLogger(DaoAbstract.class);

	protected void normalExceptionHandling(Transaction tx, Exception exception) {
		logger.error(exception.getMessage());
		if (tx != null && tx.isActive()) {
			try {
				tx.rollback();
			} catch (HibernateException hibernateException) {
				logger.error(hibernateException.getMessage());
			}
		}
	}

	protected void finalExceptionHandling(Session session) {
		if (session != null && session.isOpen()) {
			session.close();
		}
	}

	protected Session getSession() {		
		Session currentSession = HibernateUtil.getSessionFactory().getCurrentSession(); 		
		if(currentSession.isOpen()){
			logger.debug("using current session");
			return currentSession;			
		}
		logger.debug("using a new session");
		return HibernateUtil.getSessionFactory().openSession();
	}

	@SuppressWarnings("unchecked")
	protected List<Object> getListOfResultsFromNamedQuery(String namedQuery,
			int maxResults) {
		Session session = getSession();
		Transaction tx = null;
		List<Object> resultList = Collections.EMPTY_LIST;

		try {
			tx = session.beginTransaction();
			Query query = session.getNamedQuery(namedQuery);
			if (maxResults > 0) {
				query.setMaxResults(maxResults);
			}
			resultList = query.list();
			tx.commit();
			return resultList;
		} catch (Exception exception) {
			normalExceptionHandling(tx, exception);
		} finally {
			finalExceptionHandling(session);
		}
		return resultList;
	}

	protected List<Object> getListOfResultsFromNamedQuery(String namedQuery) {
		return getListOfResultsFromNamedQuery(namedQuery, 0);
	}

	protected Object getObjectById(String className, long id) {
		Session session = getSession();
		Transaction tx = null;
		Object object = null;

		try {
			tx = session.beginTransaction();
			object = session.get(className, new Long(id));
			tx.commit();
		} catch (Exception exception) {
			normalExceptionHandling(tx, exception);
		} finally {
			finalExceptionHandling(session);
		}

		return object;
	}

	protected boolean saveOrUpdate(Object object) {
		Session session = getSession();
		Transaction tx = null;

		try {
			tx = session.beginTransaction();
			
			session.saveOrUpdate(object);
			tx.commit();
			return true;

		} catch (Exception exception) {
			normalExceptionHandling(tx, exception);
		} finally {
			finalExceptionHandling(session);
		}
		return false;
	}
	
	
	protected boolean delete(Object object) {
		Session session = getSession();
		Transaction tx = null;

		try {
			tx = session.beginTransaction();
			session.delete(object);
			tx.commit();
			return true;

		} catch (Exception exception) {
			normalExceptionHandling(tx, exception);
		} finally {
			finalExceptionHandling(session);
		}

		return false;
	}
}
