package com.minlinuxserver.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import junit.framework.TestCase;

public class QueryStringBuilderTest extends TestCase {
	
	private static Logger logger = LoggerFactory
	.getLogger(QueryStringBuilderTest.class);
	
	private Map<String, String[]> keyValuePairs = null;
	private Map<String, String[]> testKeyValuePairs = new HashMap<String, String[]>();
	private static final String  testBaseUrl = "http://www.minlinuxserver.com";
	private static final String QuestionMark = "?";
	private static final String EqualsSign = "=";
	private static final String Ampersand = "&";
	
	public static final String key1 = "key1";
	public static final String key2 = "key2";
	public static final String complexKey = "1 3 5åäö";
	public static final String complexValue = "1 2 3,456åäö";
	public static final String keyWithMoreThanOneValue = "keyWithMoreThanOneValue";
	
	
	public QueryStringBuilderTest(String name) {
		super(name);
		
		String[] simpleValue1 = {"simple1Value1"};
		String[] simpleValue2 = {"simple1Value2"};
		String[] values = {"value0","value1","value2"};
		
		testKeyValuePairs.put(key1, simpleValue1);
		testKeyValuePairs.put(key2, simpleValue2);
		testKeyValuePairs.put(keyWithMoreThanOneValue, values);
		
		
	}
	
	@Override
	protected void setUp() throws Exception {
		 keyValuePairs = new HashMap<String, String[]>();
	}
	
	@Override
	protected void tearDown() throws Exception {
		keyValuePairs = null;
	}
	
	public void testQueryStringBuilderEmptyMap() throws Exception {		
		
		assertEquals(testBaseUrl, new QueryStringBuilder(testBaseUrl).buildUrl());		
	}
	
	public void testQueryStringBuilderOneEntryMap() throws Exception {
		
		keyValuePairs.put(key1, testKeyValuePairs.get(key1));
	
		String output =  new QueryStringBuilder(testBaseUrl).appendKeyAndValue(key1, testKeyValuePairs.get(key1)).buildUrl();
		
		assertTrue(output.contains(queryStringKeyValue(key1, testKeyValuePairs.get(key1), 0)));
		
		assertTrue(output.contains(testBaseUrl+QuestionMark));
		
		
	}
	
	public void testQueryStringBuilderTwoEntryMap() throws Exception {
		
		keyValuePairs.put(key1, testKeyValuePairs.get(key1));
		keyValuePairs.put(key2, testKeyValuePairs.get(key2));
		
		QueryStringBuilder queryStringBuilder = new QueryStringBuilder(testBaseUrl);
		
		queryStringBuilder.appendKeyAndValue(key1, testKeyValuePairs.get(key1)).appendKeyAndValue(key2, testKeyValuePairs.get(key2));
		
		String output =   queryStringBuilder.buildUrl();
		
		assertTrue(output.contains(queryStringKeyValue(key1, testKeyValuePairs.get(key1), 0)));
		assertTrue(output.contains(queryStringKeyValue(key2, testKeyValuePairs.get(key2), 0)));
		
		assertTrue(output.contains(testBaseUrl+QuestionMark));		
		assertTrue(output.contains(Ampersand));
	}

	public void testQueryStringBuilderOneKeyWithThreeValuesEntryMap() throws Exception {
		
		keyValuePairs.put(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue));
		keyValuePairs.put(key1, testKeyValuePairs.get(key1));
		
		QueryStringBuilder queryStringBuilder = new QueryStringBuilder(testBaseUrl);
		
		queryStringBuilder.appendKeyAndValue(key1, testKeyValuePairs.get(key1)).appendKeyAndValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue));
		
		String output =  queryStringBuilder.buildUrl();
		logger.debug(output);
		
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 0)));
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 1)));
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 2)));
		
		assertTrue(output.contains(testBaseUrl+QuestionMark));		
		assertTrue(output.contains(Ampersand));		
	}
	
	public void testQueryStringBuilderOneKeyWithMap() throws Exception {
		
		keyValuePairs.put(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue));
		keyValuePairs.put(key1, testKeyValuePairs.get(key1));
		
		QueryStringBuilder queryStringBuilder = new QueryStringBuilder();
		
		queryStringBuilder.appendMap(keyValuePairs);
		
		String output =  queryStringBuilder.buildUrl();
		logger.debug(output);
		
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 0)));
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 1)));
		assertTrue(output.contains(queryStringKeyValue(keyWithMoreThanOneValue, testKeyValuePairs.get(keyWithMoreThanOneValue), 2)));
		
		assertTrue(output.contains(QuestionMark));		
		assertTrue(output.contains(Ampersand));		
	}
	
	public void testQueryStringBuilderWithComplexKeysAndValue(){
		
		QueryStringBuilder queryStringBuilder = new QueryStringBuilder();
		
		queryStringBuilder.appendKeyAndValue(complexKey, complexValue);
		
		String output =  queryStringBuilder.buildUrl();
		logger.debug(output);
		
		assertTrue(output.contains(queryStringKeyValue(complexKey, complexValue)));
		
		assertTrue(output.contains("+"));
		assertTrue(output.contains("%"));
		
	}
	
	private String queryStringKeyValue(String key, String value){
		return urlEncode(key)+EqualsSign+urlEncode(value);
	}
	
	private String queryStringKeyValue(String key, String[] values, int index){
		return urlEncode(key)+EqualsSign+urlEncode(values[index]);
	}
	
	private String urlEncode(String input){
		try {
			return URLEncoder.encode(input,"UTF-8");
		} catch (UnsupportedEncodingException e) {
			return "";
		}
	}
	
}

